package yrpg.gmscm.rc.rcd.tools;


import org.joda.time.LocalDate;
import org.joda.time.LocalDateTime;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;

public class GetDateUtils {


    private final static int[] MONTH_OF_QUARTER_END = {3, 6, 9, 12};
    private final static int[] MONTH_OF_HALF_YEAR_END = {6, 12};

    /**
     * 获取季度最后一天
     *
     * @param today
     * @return
     */
    public static Date getEndDayOfQuarter(Date today) {
        LocalDate localDate = new LocalDate(today);
        int monthOfYear = localDate.getMonthOfYear();

        while (!isQuarterMonth(monthOfYear)) {
            localDate = localDate.plusMonths(1);
            monthOfYear = localDate.getMonthOfYear();
        }
        return localDate.dayOfMonth().withMaximumValue().toDate();
    }

    /**
     * 获取当月最后一天
     *
     * @param today
     * @return
     */
    public static Date getEndDayOfMonth(Date today) {
        LocalDate localDate = new LocalDate(today);
        return localDate.dayOfMonth().withMaximumValue().toDate();
    }

    /**
     * 获取半年最后一天
     *
     * @param today
     * @return
     */
    public static Date getEndDayOfHalfYear(Date today) {
        LocalDate localDate = new LocalDate(today);
        int monthOfYear = localDate.getMonthOfYear();

        while (!isHalfYearMonth(monthOfYear)) {
            localDate = localDate.plusMonths(1);
            monthOfYear = localDate.getMonthOfYear();
        }
        return localDate.dayOfMonth().withMaximumValue().toDate();
    }

    /**
     * 判断当前月是否为季度最后一个月
     *
     * @param month
     * @return
     */
    private static boolean isQuarterMonth(int month) {
        return Arrays.stream(MONTH_OF_QUARTER_END).anyMatch(t -> t == month);
    }

    /**
     * 判断当前月是否为半年最后一个月
     *
     * @param month
     * @return
     */
    private static boolean isHalfYearMonth(int month) {
        return Arrays.stream(MONTH_OF_HALF_YEAR_END).anyMatch(t -> t == month);
    }


    /**
     * 取日期的最后时间 即 23:59:59
     *
     * @param date
     * @return
     */
    public static Date toDayEnd(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        Date d = calendar.getTime();
        return d;
    }

    /**
     * 获取当天开始时间
     *
     * @param now
     * @return
     */
    public static Date toDayStart(Date now) {
        return new LocalDate(now).toDateTimeAtStartOfDay().toDate();
    }

    /**
     * 获取第二天开始时间
     *
     * @param now
     * @return
     */
    public static Date toNextDayStart(Date now) {
        return new LocalDate(now).plusDays(1).toDateTimeAtStartOfDay().toDate();
    }

    /**
     * 判断是否当月最后一天
     *
     * @param now
     * @return
     */
    public static boolean isMonthEndDay(Date now) {
        LocalDate localDate = new LocalDate(now);
        int nowDayOfMonth = localDate.getDayOfMonth();
        int dayOfMonth = localDate.dayOfMonth().withMaximumValue().getDayOfMonth();
        return nowDayOfMonth == dayOfMonth;
    }

    /**
     * 获取当月第一天
     *
     * @param date
     * @return
     */
    public static Date getStartDayOfMonth(Date date) {
        LocalDate localDate = new LocalDate(date);
        return localDate.dayOfMonth().withMinimumValue().toDate();
    }

    /**
     * 获取当月最后一天23点
     *
     * @param today
     * @return
     */
    public static Date getEndDayEndTimeOfMonth(Date today) {
        LocalDateTime localDate = new LocalDateTime(today);
        return localDate.dayOfMonth().withMaximumValue().plusDays(1).minusSeconds(1).toDate();
    }

    /**
     * 前 n 天
     *
     * @return
     */
    public static Date getLastnDate(int n) {
        Calendar cd = Calendar.getInstance();
        cd.setTime(new Date(System.currentTimeMillis()));
        cd.add(Calendar.DATE, -n);// 减少一天
        return cd.getTime();
    }

    /**
     * 日期加一个月
     *
     * @param date
     * @return
     */
    public static Date addOneMonth(Date date) {
        LocalDateTime localDate = new LocalDateTime(date);
        LocalDateTime localDateTime = localDate.minusMonths(-1);
        Date date1 = localDateTime.toDate();
        return date1;
    }

    /**
     * 日期减一个月
     *
     * @param date
     * @return
     */
    public static Date subtractOneMonth(Date date) {
        LocalDateTime localDate = new LocalDateTime(date);
        LocalDateTime localDateTime = localDate.minusMonths(1);
        Date date1 = localDateTime.toDate();
        return date1;
    }

    /**
     * 判断时间是否处于制定时间月份
     */
    public static boolean isBetweenMonth(Date sourceDate, Date monthDate) {
        LocalDate monthLocalDate = new LocalDate(monthDate);
        LocalDate sourceLocalDate = new LocalDate(sourceDate);
        LocalDate minDate = monthLocalDate.dayOfMonth().withMinimumValue();
        LocalDate maxDate = monthLocalDate.dayOfMonth().withMaximumValue();
        if (minDate.isBefore(sourceLocalDate) && maxDate.isAfter(sourceLocalDate)) {
            return true;
        }
        return false;
    }

    /**
     * 判断当前区间是否包含指定月份
     */
    public static boolean isBetweenMonth(Date startDate, Date endDate, Date monthDate) {
        LocalDate monthLocalDate = new LocalDate(monthDate);
        LocalDate startLocalDate = new LocalDate(startDate);
        LocalDate endLocalDate = new LocalDate(endDate);
        LocalDate minDate = monthLocalDate.dayOfMonth().withMinimumValue();
        LocalDate maxDate = monthLocalDate.dayOfMonth().withMaximumValue();
        if (minDate.isBefore(startLocalDate) || endLocalDate.isAfter(maxDate)) {
            return true;
        }
        return false;
    }

}
